%Simulation code for Table 3 of Kline and Santos (2011)

clear

randn('seed',1234)
rand('seed',1234)


R=10000; %number of simulations
R2=200; %bootstrap reps




%read the data

data=csvread('~/wild/hate/Census/data.txt');
Nfull=length(data);
Yfull=data(:,1);
Xfull=[ones(Nfull,1) data(:,[5 2 3 4])]; %order black dummy first



b=Xfull\Yfull;  %get the population regression coefficient

results=NaN(4,6,2);



for n=1:4

N=2^(n-1)*50; %sample size    
    
%make wild bootstrap weights
%obeying E[W]=0, E[W^2]=1, E[W^3]=0
W=gaminv(rand(N,R),4,1/2)-2; %gamma weights
Wrad=2*(rand(N,R)<.5)-1; %rad weights
pos=rand(N,R)<.7236068;
Wmammen=-.61803399*pos + (1-pos)*(1+.61803399); %Mammen weights

K=size(Xfull,2);
dfk=N/(N-K);


pairs=NaN(N,R,R2);
for i=1:R
    for j=1:R2
        pairs(:,i,j)=randsample(N,N,1); %random pairs draw
    end
end


draws=NaN(N,R);
for i=1:R
    draws(:,i)=randsample(Nfull,N,1);
end



%initialize results registers
rejectwhite=NaN(R,1);
rejectwild=NaN(R,1);
rejectwildrad=NaN(R,1);
rejectwildmammen=NaN(R,1);
rejectpairs=NaN(R,1);

rejectwhite1=NaN(R,1);
rejectwild1=NaN(R,1);
rejectwildrad1=NaN(R,1);
rejectwildmammen1=NaN(R,1);
rejectpairs1=NaN(R,1);


for i=1:R

    %sample the data  
      
    X=Xfull(draws(:,i),:);
    Y=Yfull(draws(:,i),:);

    %compute full sample stat
    xx=X'*X;
    if rank(xx)==K        
    xxi=inv(xx);
    [beta,ci,resid]=regress(Y,X);
        
    bigsig=xxi*X'*diag(resid.^2)*X*xxi*dfk;
    sig=sqrt(bigsig(2,2));
    tstat=(beta(2)-b(2))/sig; %full sample t-stat (heteroscedastic)
        
    
    
    %bootstraps
    wildt=NaN(R2,1);
    wildradt=NaN(R2,1);        
    wildmamment=NaN(R2,1);
    pairst=NaN(R2,1);
    
    %Wild and pairs bootstraps
    parfor r=1:R2 
	%Gamma
        Ystar=X*beta+resid.*W(:,r);
        [wildbeta,ci,wildresid]=regress(Ystar,X);
        wildbigsig=xxi*X'*diag(wildresid.^2)*X*xxi*dfk;
        wildsig=sqrt(wildbigsig(2,2));
        wildt(r,1)=(wildbeta(2)-beta(2))/wildsig; %wild t-stat (centered around full sample)

	%Rad
        Ystar=X*beta+resid.*Wrad(:,r);
        [wildbeta,ci,wildresid]=regress(Ystar,X);
        wildbigsig=xxi*X'*diag(wildresid.^2)*X*xxi*dfk;
        wildsig=sqrt(wildbigsig(2,2));
        wildradt(r,1)=(wildbeta(2)-beta(2))/wildsig; %wild Rad t-stat (centered around full sample)

        
	%Mammen
        Ystar=X*beta+resid.*Wmammen(:,r);
        [wildbeta,ci,wildresid]=regress(Ystar,X);
        wildbigsig=xxi*X'*diag(wildresid.^2)*X*xxi*dfk;
        wildsig=sqrt(wildbigsig(2,2));
        wildmamment(r,1)=(wildbeta(2)-beta(2))/wildsig; %wild Mammen t-stat (centered around full sample)

        
	%Pairs
        Ypairs=Y(pairs(:,i,r),1);
        Xpairs=X(pairs(:,i,r),:);
        xxpairs=Xpairs'*Xpairs;
        if rank(xxpairs)==K
        xxipairs=inv(xxpairs);
        [pairsbeta,ci,pairsresid]=regress(Ypairs,Xpairs);
        pairsbigsig=xxipairs*Xpairs'*diag(pairsresid.^2)*Xpairs*xxipairs*dfk;
        pairssig=sqrt(pairsbigsig(2,2));
        pairst(r,1)=(pairsbeta(2)-beta(2))/pairssig; %pairs t-stat (centered around full sample)
        else
            pairst(r,1)=NaN;
        end
    end
    
    critT=quantile(abs(wildt),.95);
    critTrad=quantile(abs(wildradt),.95);
    critTmammen=quantile(abs(wildmamment),.95);
    critTpairs=quantile(abs(pairst),.95);
    

    critT1=quantile(wildt,.05);
    critTrad1=quantile(wildradt,.05);
    critTmammen1=quantile(wildmamment,.05);
    critTpairs1=quantile(pairst,.05);
       

    rejectwhite(i)=abs(tstat)>1.96;
    rejectwild(i)=abs(tstat)>critT;    
    rejectwildrad(i)=abs(tstat)>critTrad;
    rejectwildmammen(i)=abs(tstat)>critTmammen;
    rejectpairs(i)=abs(tstat)>critTpairs;

    rejectwhite1(i)=tstat<-1.65;
    rejectwild1(i)=tstat<critT1;    
    rejectwildrad1(i)=tstat<critTrad1;
    rejectwildmammen1(i)=tstat<critTmammen1;
    rejectpairs1(i)=tstat<critTpairs1;

    
    end
end

results(n,:,1)=[N,mean(rejectwhite(~isnan(rejectwhite))),mean(rejectwild1(~isnan(rejectwild))),...
mean(rejectwildrad(~isnan(rejectwildrad))),mean(rejectwildmammen(~isnan(rejectwildmammen))),mean(rejectpairs(~isnan(rejectpairs)))];

results(n,:,2)=[N,mean(rejectwhite1(~isnan(rejectwhite1))),mean(rejectwild1(~isnan(rejectwild1))),...
mean(rejectwildrad1(~isnan(rejectwildrad1))),mean(rejectwildmammen1(~isnan(rejectwildmammen1))),mean(rejectpairs1(~isnan(rejectpairs1)))];


end




results


save results_census results

